from .api import do_get, do_post, authenticate, launch_tool, get_results, check_status, load_results
from .hubxml import get_driver, extract_results, extract_all_results
from IPython.display import clear_output
from floatview import Floatview
from ipywidgets import Output, Tab, ToggleButton, Text, Dropdown, IntText, VBox, HBox, Accordion, Button
import time, threading

class Nanohubtool():

    
    
    widget_output = None    
    session = None
    def __init__(self, credentials, **kwargs):
        self.options = {}
        self.experiments = []
        self.options = {}
        self.window = None
        self.tab = None

        self.debug = kwargs.get('debug', None)
        self.modal = kwargs.get('modal', True)
        self.only_subsets = kwargs.get('only_subsets', False)
        self.output_cont = Output()
        self.output_cont.layout.width = '100%'
        self.options_cont = Output()
        self.margins_cont = Output()
        self.widget_output = Output()
        self.options_but = Button(description="Run ...", tooltip='Run exploration', icon='check', disable=False)
        self.options_but.on_click(lambda b : Nanohubtool.exploreOptions(self))
        self.option_tab = Tab()
        self.option_tab.children = [self.options_cont]
        self.option_tab.set_title(0, "Parameters")
        self.option_tab.layout.display = None

        try : 
            self.session = Nanohubsession(credentials)
        except ConnectionError as ce:
            print ("Invalid Authentication, server returned: " + str(ce))
        except:
            print ("Invalid Authentication")
        
        

        self.options_check = ToggleButton(value = True, description="parameters", icon='cog')
        self.options_check.observe(lambda v:self.showWidget(v["new"]), names='value')

        self.tab = HBox()
        self.tab.children = [self.option_tab, self.output_cont]
        if (self.window == None) :
            if (self.modal == True):
                title = kwargs.get('title', "")
                mode = kwargs.get('mode', "")
                self.window = Floatview(title = title, mode = mode)  
            else:
                self.window = Output()
                display(self.window)
        self.displayWindow()
           
    def display(self):
        self.displayOutput()
        self.displayOptions()

    def addExperiment(self, experiment):
        self.experiments.append(experiment)
        
    def exploreOptions(self):
        parameters = {}
        for key, val in self.options.items():
            parameters[key] = val.value
            self.options[key].value = ''
        experiment = { 'parameters':parameters }
        self.addExperiment(experiment)

        
    def displayOutput(self):   
        with self.output_cont:
            clear_output()
            display(self.widget_output)

    def displayWindow(self):   
        with self.window:
            clear_output()
            display(self.options_check) 
            display(self.tab)


    def displayOptions(self): 
        with self.options_cont:
            clear_output()
            if len(self.options) > 0:
                for key, option in self.options.items():
                    display(option)
            display(self.options_but)
            
                
    def showWidget(self, show):
        if show:
            self.option_tab.layout.display = None
        else:
            self.option_tab.layout.display = "None"

class Nanohubsession():
    credentials = {}
    headers = {}
    authenticated = False
    def __init__(self, credentials, **kwargs):
        self.credentials = credentials
        self.validateSession()
            
    def validateSession(self):
        if self.authenticated == True:
            pass; #TODO: validate session is active;
        if self.authenticated == False:
            try :
                self.headers = authenticate(self.credentials)
                self.authenticated = True
            except AttributeError:
                self.authenticated = False        
                raise ConnectionError("Authentication Token not found")
            except ConnectionError as ce:
                self.authenticated = False        
                raise ce
            except:
                self.authenticated = False        
                raise ConnectionError("invalid Authentication")
                
    def getSessionId(self, tool, tool_inputs):
        self.validateSession() 
        if (self.authenticated):
            driver_json = get_driver(tool, tool_inputs, self.headers)
            session_id = launch_tool(driver_json, self.headers)
            return session_id
        return None
            
    def checkStatus(self, session_id):
        self.validateSession() 
        if (self.authenticated):
            return check_status({'session_num': session_id}, self.headers)
        return {}
        
    def getResults(self, session_id, run_file):
        self.validateSession() 
        results_json = {
            'session_num': session_id,
            'run_file': run_file
        }
        if (self.authenticated):
            return load_results(results_json, self.headers)
        return ''

        
        
class setInterval :
    def __init__(self,interval,action) :
        self.interval=interval
        self.action=action
        self.stopEvent=threading.Event()
        thread=threading.Thread(target=self.__setInterval)
        thread.start()

    def __setInterval(self) :
        nextTime=time.time()+self.interval
        while not self.stopEvent.wait(nextTime-time.time()) :
            nextTime+=self.interval
            self.action()

    def cancel(self) :
        self.stopEvent.set()        

    


#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Project Jupyter.
# Distributed under the terms of the Modified BSD License.

from .nanohubtools import Nanohubtool
from .qdottools import Qdotexplorer
from .api import *
from .hubxml import *


from ._version import __version__, version_info




"""
Copyright 2019 HUBzero Foundation, LLC.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

HUBzero is a registered trademark of Purdue University.

Authors:
Benjamin P. Haley, Purdue University (bhaley@purdue.edu)
Daniel Mejia (denphi), Purdue University (denphi@denphi.com)
"""

try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode

try:
    from urllib2 import urlopen, Request, HTTPError
except ImportError:
    from urllib.request import urlopen, Request, HTTPError

import sys, json, time
import requests
import json

url = r'https://nanohub.org/api'
sleep_time = 1.5

def do_get(url, path, data, hdrs):
    request = requests.get('{0}/{1}'.format(url, path) , data=data, headers=hdrs)
    return request.json()    

def do_post(url, path, data, hdrs):
    request = requests.post('{0}/{1}'.format(url, path) , data=data, headers=hdrs)
    return request.json()
    
def validate_request (auth_json):
    if 'errors' in auth_json:
        raise ConnectionError(json.dumps(auth_json['errors']))


def authenticate(auth_data):
    auth_json = do_post(url, 'developer/oauth/token', auth_data, hdrs={})
    validate_request(auth_json)
    if 'access_token' in auth_json:
        return {'Authorization': 'Bearer {}'.format(auth_json['access_token'])}
    else :
        raise AttributeError('access_token')
        
def launch_tool(driver_json, headers):
    """Start a tool session; return the session id"""
    run_json = do_post(url, 'tools/run', driver_json, headers)
    if 'session' in run_json:
        return run_json['session']
    else:
        msg = 'launch_tool failed ({0}): {1}\n'.format(run_json['code'], run_json['message'])
        sys.stderr.write(msg)
        sys.exit(1)
        
def check_status(session_json, headers):
    """Start a tool session; return the session id"""
    status_json = do_post(url, 'tools/status', session_json, headers)
    return status_json
        
def get_results(session_id, headers):
    """Wait for the tool session to finish; return the run XML"""
    status_data = {'session_num': session_id}
    while True:
        time.sleep(sleep_time)
        status_json = do_post(url, 'tools/status', status_data, headers)
        if 'finished' in status_json : 
            if status_json['finished'] is True:
                break
    time.sleep(sleep_time)  # let the DB update
    result_data = {
        'session_num': session_id,
        'run_file': status_json['run_file']
    }
    result_json = do_post(url, 'tools/output', result_data, headers)
    return result_json['output']

    
def load_results(run_json, headers):
    result_json = do_post(url, 'tools/output', run_json, headers)
    if 'output' in result_json:
        return result_json['output'];
    else:
        return '';



from .nanohubtools import Nanohubtool, setInterval
from plotly.graph_objs import FigureWidget
from ipywidgets import IntText, Text, Output, Tab, Button, Accordion, GridBox, Layout, ButtonStyle, Label
from IPython.display import clear_output
from .hubxml import extract_all_results
import numpy as np
import xml.etree.ElementTree as ET
from floatview import Floatview
from threading import Lock

import copy
from base64 import b64decode, b64encode
import zlib
import time

class Qdotexplorer (Nanohubtool):
    
    def __init__(self, credentials, **kwargs):
        Nanohubtool.__init__(self, credentials, **kwargs)
        #self.options['nsample'] = IntText(description = '# samples:', value = 2)
        #self.options['nsample'].observe(lambda v:self.UpdateTraces({'line.width':v['new']}), names='value')                
        self.experiment_container = None
        self.debug_output = Output()
        
        self.parameters = kwargs.get('parameters', [])
        self.units = kwargs.get('units', ['' for i in self.parameters])
        self.sleep_time = 5
        
        while (len(self.parameters) > len(self.units)):
            self.units.append('')
            
        for param in self.parameters:
            self.options[param] = Text(value='',placeholder=param,description='',disabled=False)

        self.updateRender()
        self.inter = setInterval(self.sleep_time, lambda a=self : Qdotexplorer.updateExperiments(a))

        def __del__(self):
            self.inter.cancel()
        
    def createToolWidget(self):    
        self.widget_output.clear_output()
        self.experiment_container = Tab()
        self.experiment_container.children = []
        with self.widget_output:
            display(self.experiment_container)
            
    def createToolWidget(self):    
        self.widget_output.clear_output()
        self.experiment_container = Tab()
        self.experiment_container.children = []
        with self.widget_output:
            display(self.experiment_container)
            display(self.debug_output)
        with self.debug_output:
            print("testing")
            


    def addExperiment(self, experiment):
        experiment['session_id'] = None
        experiment['complete'] = False
        experiment['results'] = ''
        experiment['output'] = Output()
        experiment['status'] = Label(value='New',layout=Layout(width='auto'))
        self.experiments.append(experiment)      
        children = list(self.experiment_container.children)
        children.append(experiment['output'])
        self.experiment_container.children=children
        len_exp = len(self.experiment_container.children)
        self.experiment_container.set_title(len_exp-1, "New")
        self.displayExperiment(experiment)
            
    def updateExperiments(self):
        lock = Lock()
        if (lock.acquire(False)):
            if (self.session):    
                for i, experiment in enumerate(self.experiments):
                    if experiment['session_id'] == None:
                        try:
                            experiment['session_id'] = self.session.getSessionId('qdot', experiment['parameters'])
                            self.experiment_container.set_title(i, experiment['session_id'])
                        except:
                            experiment['session_id'] = None
                    else: 
                        if experiment['complete'] == False:
                            status = self.session.checkStatus(experiment['session_id'])                                  
                            if 'success' in status and status['success']: 
                                if 'status' in status:
                                    if len(status['status']) > 0:
                                        experiment['status'].value = str(status['status'][0])
                                if 'finished' in status and 'run_file' in status:
                                    if status['finished'] and status['run_file'] != "":
                                        experiment['complete'] = True                                    
                                        with experiment['output']:
                                            but = Button(description='Load results', icon='check', disable=False)
                                            but.on_click(lambda a, b=self, c=experiment, d=status['run_file'] : Qdotexplorer.loadResults(b,c,d))
                                            display(but)
            lock.release()
    
    def loadResults(self, experiment, run_file):
        xml = self.session.getResults(experiment['session_id'], run_file)            
        xml = ET.fromstring(xml)
        experiment['results'] = xml.find('output')
        self.displayExperiment(experiment)

                
    
    
    def loadExperiment(self, session_id):
        experiment = {'parameters':{}}
        if self.session is not None:
            status = self.session.checkStatus(session_id)
            experiment['session_id'] = session_id
            experiment['complete'] = False            
            experiment['results'] = ''
            experiment['output'] = Output()
            experiment['status'] = Label(value='Complete',layout=Layout(width='auto'))
            if 'finished' in status:
                if status['finished']:
                    experiment['complete'] = True            
                    if 'run_file' in status:
                        self.loadResults(experiment, status['run_file'])

            self.experiments.append(experiment)                        
            children = list(self.experiment_container.children)
            children.append(experiment['output'])
            self.experiment_container.children=children
            len_exp = len(self.experiment_container.children)
            self.experiment_container.set_title(len_exp-1, experiment['session_id'])
            self.displayExperiment(experiment)
            

    def displayExperiment(self, experiment):

        params_b = []
        for k, v in experiment['parameters'].items():
            params_b.append(Button(description=k,layout=Layout(width='auto'),style=ButtonStyle(button_color='lightblue')))
            params_b.append(Label(value=v,layout=Layout(width='auto')))
        params_b.append(Button(description='Status',layout=Layout(width='auto'),style=ButtonStyle(button_color='lightblue')))
        params_b.append(experiment['status'])
        grid = GridBox(children=params_b, layout=Layout(grid_template_columns='50% 50%'))            
        out_param = Output()
    
        if (experiment['complete']):
            out_curves = Output()
            out_volumes = Output()    
            accordion = Accordion(children=[out_param, out_curves, out_volumes])
            accordion.set_title(0, 'Parameters')
            accordion.set_title(1, 'Curves')
            accordion.set_title(2, 'Volumes')
                
            experiment['output'].clear_output()
            with experiment['output']:
                display(accordion)
            with out_param:
                display(grid)
            with out_curves:            
                curves = experiment['results'].findall('curve')
                self.actions = [None for c in curves]
                for i in range(len(curves)):
                    el = curves[i]
                    but = Button(description=el.find("about").find("label").text, icon='check', disable=False)
                    but.on_click(lambda a, b=self, c=[curves[i]] : Qdotexplorer.plotXY(b,c))
                    display(but)
            with out_volumes:                            
                component = experiment['results'].find("table").find("data").text
                data = component.split('\n')
                data = data[0:-1]
                for i,v in enumerate(data):
                    component = experiment['results']
                    component = self.getData(component, 'sequence') 
                    component = self.getData(component[0], 'element') 
                    component = self.getData(component[i], 'field') 
                    component = self.getData(component[0], 'component') 
                    but = Button(description=v, icon='check', disable=False)
                    but.on_click(lambda a, b=self, c=v, d=component : Qdotexplorer.plotVTK(b,c,d))
                    display(but)
        else:
            accordion = Accordion(children=[out_param])
            accordion.set_title(0, 'Parameters')
            with experiment['output']:
                display(accordion)
            with out_param:
                display(grid)

                
    

    def getData(self, field, container):
        component = field.findall(container)
        return component

    def getXY(self, field, container):
        list_v = []
        component = self.getData(field, container) 
        for obj in component:
            xy = obj.find("xy").text
            list_v.append(xy)
        return list_v

    def getText(self, obj, fields):
        objf = obj
        text = ''
        try:
            for field in fields:
                objf = objf.find(field)
            text = objf.text
        except: 
            text = ''    
        return text
  
    def getVTK(self, component, container):
        list_v = []
        for obj in component:
            vtk = obj.find("vtk").text
            compressed_header = '@@RP-ENC:zb64\n'
            hlen = len(compressed_header)
            vtk = vtk[hlen:-1]
            datavtk = b64decode(vtk)
            list_v.append(zlib.decompress(datavtk, zlib.MAX_WBITS | 32))
        return list_v
  
    def plotVTK( self, title, component ):
        traces = []
        
        component = self.getVTK(component , 'component')
        datavtk = component[0]
        npdata = np.array(datavtk.splitlines())
        dimensions = npdata[4].split()
        dimensions = [int(dimensions[v]) for v in range (1,len(dimensions)) ]
        spacing = npdata[5].split()
        spacing = [float(spacing[v]) for v in range (1,len(spacing)) ]
        origin = npdata[6].split()
        origin = [float(origin[v]) for v in range (1,len(origin)) ]
        values = [np.fromstring(npdata[i], dtype=float, sep=" ") for i in range(10, len(npdata))]
        self.V = np.concatenate(values)
        x = 2
        y = 1
        z = 0
        X,Y,Z = np.mgrid[origin[x]:(spacing[x]*dimensions[x]):spacing[x], origin[y]:(spacing[y]*dimensions[y]):spacing[y], origin[z]:(spacing[z]*dimensions[z]):spacing[z]]
        self.X = X.flatten().tolist()
        self.Y = Y.flatten().tolist()
        self.Z = Z.flatten().tolist()
        min_val = np.amin(self.V)
        max_val = np.amax(self.V) 
        ncontours = 12
        n_value = np.linspace(min_val, max_val, num=ncontours + 2);

        trace = {
            "type": "volume",
            "x": self.X,
            "y": self.Y,
            "z": self.Z,
            #'marker' : {
            #    'color' : self.V,
            #    'opacity' : 0.35,
            #},
            "value": self.V,
            #"isomin": n_value[1],
            #"isomax": n_value[len(n_value)-2],
            "spaceframe": { "show": False, "fill": 0.0  },
            "surface": { "show": True, "fill": 1.0, "count":ncontours},
            "caps": {
              "x": { "show": True, "fill": 1.0 },
              "y": { "show": True, "fill": 1.0 },
              "z": { "show": True, "fill": 1.0 }
            },      
            "slices": {
              "x": { "show": False, },
              "y": { "show": False, },
              "z": { "show": False, }
            },      
            "colorscale": "Viridis",
            "reversescale": False,
            "opacity":0.8,
            "opacityscale":"max",
            "lighting" :{
                "ambient" : 1.0,
                "diffuse" : 0.0,
                "specular" : 0.0,
                "roughness" : 0.0,
                "fresnel" : 0.0,
            },
            "lightposition":{
                "x" : 10000,
                "y" : 10000,
                "z" : 0,
            }      
        }
        traces.append(trace)
        layout = {
        } 
        
        fig = FigureWidget({
            'data': traces,
            'layout': layout
        })
        out = Floatview(title=title, mode = 'split-bottom')
        with out:
            display(fig)        

  
    def plotXY(self, fields):
        traces = []
        for field in fields:
            component = self.getXY(field, 'component')
            label = self.getText(field, ["about","label"])
            title = self.getText(field, ["about","group"])
            xaxis = self.getText(field, ["xaxis","label"])
            xunits = self.getText(field, ["xaxis","units"])
            xscale = self.getText(field, ["xaxis","scale"])
            if xscale == "":
                xscale = "linear"
                yaxis = self.getText(field, ["yaxis","label"])
                yunits = self.getText(field, ["yaxis","units"])
                yscale = self.getText(field, ["yaxis","scale"])    
            if yscale == "":
                yscale = "linear"
            for obj in component:
                xy = obj.strip()
                xy = np.array(xy.splitlines())
                xy = xy[(xy != '')]
                xy = [np.fromstring(xy[i], dtype=float, sep=" ") for i in range(len(xy))]
                xy = np.concatenate(xy)
                xy = xy.reshape(int(len(xy)/2),2)
                trace1 = {
                    'type' : 'scatter',
                    'x' : xy[:,0],
                    'y' : xy[:,1],
                    'mode' : 'lines',
                    'name' : label
                }
                traces.append(trace1)
        
        layout = {
            'title' : title,
            'xaxis' : {
                'title' : xaxis + " [" + xunits + "]",
                'type' : xscale,
                'autorange' : True,
                'exponentformat' :  "e",
            },
            'yaxis' : {
                'title' : yaxis + " [" + yunits + "]",
                'type' : yscale,
                'autorange' : True,
                'exponentformat' : "e"
            },
            'legend' : { 'orientation' : 'h'},          
        }    
        fig = FigureWidget({
            'data': traces,
            'layout': layout
        })
        out = Floatview(title=title, mode = 'split-bottom')
        with out:
            display(fig)

            
    def updateRender(self):		
        self.createToolWidget()
        Nanohubtool.display(self)





"""
Manipulate XML for nanoHUB tools

Benjamin P. Haley, Purdue University (bhaley@purdue.edu)

Copyright 2017 HUBzero Foundation, LLC.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

HUBzero is a registered trademark of Purdue University.
"""

import json
import numpy as np
import xml.etree.ElementTree as et
from .api import launch_tool, get_results

# Driver template for the drivergen tool
template = """<?xml version="1.0"?>
<run>
    <input>
        <string id="toolname"><current>{0}</current></string>
        <string id="inputs"><current>{1}</current></string>
    </input>
</run>
"""

def get_driver(tool_name, inputs, headers):
    """Get the driver XML for running a nanoHUB tool with specific inputs"""
    drivergen_json = {
        'app': 'drivergen',
        'xml': template.format(tool_name, json.dumps(inputs))
    }
    session_id = launch_tool(drivergen_json, headers)
    run_xml = get_results(session_id, headers)
    xml = et.fromstring(run_xml)  # <run>
    driver_str  = '<?xml version="1.0"?>\n'
    driver_str += xml.find('./output/string/current').text
    return {'app': tool_name, 'xml': driver_str}

def extract_results(run_xml, outputs):
    """
    Parse run_xml to extract the specified output; return a dict with the 
    output label as a key.  The returned values will be floats for <number> 
    outputs, (x,y) tuples of numpy arrays for <curve> outputs, and strings
    for all other output types.
    """
    d = {}
    xml = et.fromstring(run_xml)  # <run>
    for el in xml.find('output'):
        lel = el.find('./about/label')
        if lel is not None and lel.text in outputs:
            cel = el.find('current')
            val = cel.text
            if el.tag == 'number':
                val = float(val)
            elif el.tag == 'curve':
                lines = val.split('\n')
                n = len(lines)
                x = np.zeros(n)
                y = np.zeros(n)
                for i in range(n):
                    words = lines[i].split()
                    x[i] = float(words[0])
                    y[i] = float(words[1])
                val = (x,y)
            d[lel.text] = val
    return d

def extract_all_results(run_xml):
    """
    Parse run_xml to extract the specified output; return a dict with the 
    output label as a key.  The returned values will be floats for <number> 
    outputs, (x,y) tuples of numpy arrays for <curve> outputs, and strings
    for all other output types.
    """
    d = {}
    xml = et.fromstring(run_xml)  # <run>
    for el in xml.find('output'):
        lel = el.find('./about/label')
        cel = el.find('current')
        val = cel.text
        if el.tag == 'number':
            val = float(val)
        elif el.tag == 'curve':
            lines = val.split('\n')
            n = len(lines)
            x = np.zeros(n)
            y = np.zeros(n)
            for i in range(n):
                words = lines[i].split()
                x[i] = float(words[0])
                y[i] = float(words[1])
            val = (x,y)
        d[lel.text] = val
    return d


#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

from __future__ import print_function
from glob import glob
from os.path import join as pjoin


from setupbase import (
    create_cmdclass, install_npm, ensure_targets,
    find_packages, combine_commands, ensure_python,
    get_version, HERE
)

from setuptools import setup


# The name of the project
name = 'nanohubtools'

# Ensure a valid python version
ensure_python('>=3.3')

# Get our version
version = get_version(pjoin(name, '_version.py'))

long_description = ""
with open("README.md", "r") as fh:
    long_description = fh.read()

setup_args = dict(
    name            = name,
    description     = 'A floatview output widget for JupyterLab + GlueViz Visualization with plotly',
    long_description=long_description,
    long_description_content_type="text/markdown",
    version         = version,
    scripts         = glob(pjoin('scripts', '*')),
    packages        = find_packages(),
    author          = 'Project Jupyter contributor',
    author_email    = 'denphi@denphi.com',
    url             = 'https://github.com/denphi/nanohubtools',
    license         = 'BSD',
    platforms       = "Linux, Mac OS X, Windows",
    keywords        = ['Jupyter', 'Widgets', 'IPython'],
    classifiers     = [
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Jupyter',
    ],
    include_package_data = True,
    install_requires = [
        'floatview>=0.1.11',
        'plotly>=3.8.1',
        'numpy>=1.16.0',
    ],
    extras_require = {
        'test': [
        ],
        'examples': [
            # Any requirements for the examples to run
        ],
        'docs': [
        ],
    },
    entry_points = {
    },
)

if __name__ == '__main__':
    setup(**setup_args)
